from http import HTTPStatus
from typing import Any, Dict, Optional, Union
from uuid import UUID

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.api_change_parent_item_v1_items_item_uuid_parent_new_parent_uuid_put_response_api_change_parent_item_v1_items_item_uuid_parent_new_parent_uuid_put import (
    ApiChangeParentItemV1ItemsItemUuidParentNewParentUuidPutResponseApiChangeParentItemV1ItemsItemUuidParentNewParentUuidPut,
)
from ...models.http_validation_error import HTTPValidationError
from ...types import Response


def _get_kwargs(
    item_uuid: UUID,
    new_parent_uuid: UUID,
) -> Dict[str, Any]:
    _kwargs: Dict[str, Any] = {
        "method": "put",
        "url": f"/v1/items/{item_uuid}/parent/{new_parent_uuid}",
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[
    Union[
        ApiChangeParentItemV1ItemsItemUuidParentNewParentUuidPutResponseApiChangeParentItemV1ItemsItemUuidParentNewParentUuidPut,
        HTTPValidationError,
    ]
]:
    if response.status_code == 202:
        response_202 = ApiChangeParentItemV1ItemsItemUuidParentNewParentUuidPutResponseApiChangeParentItemV1ItemsItemUuidParentNewParentUuidPut.from_dict(
            response.json()
        )

        return response_202
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[
    Union[
        ApiChangeParentItemV1ItemsItemUuidParentNewParentUuidPutResponseApiChangeParentItemV1ItemsItemUuidParentNewParentUuidPut,
        HTTPValidationError,
    ]
]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    item_uuid: UUID,
    new_parent_uuid: UUID,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Response[
    Union[
        ApiChangeParentItemV1ItemsItemUuidParentNewParentUuidPutResponseApiChangeParentItemV1ItemsItemUuidParentNewParentUuidPut,
        HTTPValidationError,
    ]
]:
    """Api Change Parent Item

     Rename exising item.

    Args:
        item_uuid (UUID):
        new_parent_uuid (UUID):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[ApiChangeParentItemV1ItemsItemUuidParentNewParentUuidPutResponseApiChangeParentItemV1ItemsItemUuidParentNewParentUuidPut, HTTPValidationError]]
    """

    kwargs = _get_kwargs(
        item_uuid=item_uuid,
        new_parent_uuid=new_parent_uuid,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    item_uuid: UUID,
    new_parent_uuid: UUID,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Optional[
    Union[
        ApiChangeParentItemV1ItemsItemUuidParentNewParentUuidPutResponseApiChangeParentItemV1ItemsItemUuidParentNewParentUuidPut,
        HTTPValidationError,
    ]
]:
    """Api Change Parent Item

     Rename exising item.

    Args:
        item_uuid (UUID):
        new_parent_uuid (UUID):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[ApiChangeParentItemV1ItemsItemUuidParentNewParentUuidPutResponseApiChangeParentItemV1ItemsItemUuidParentNewParentUuidPut, HTTPValidationError]
    """

    return sync_detailed(
        item_uuid=item_uuid,
        new_parent_uuid=new_parent_uuid,
        client=client,
    ).parsed


async def asyncio_detailed(
    item_uuid: UUID,
    new_parent_uuid: UUID,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Response[
    Union[
        ApiChangeParentItemV1ItemsItemUuidParentNewParentUuidPutResponseApiChangeParentItemV1ItemsItemUuidParentNewParentUuidPut,
        HTTPValidationError,
    ]
]:
    """Api Change Parent Item

     Rename exising item.

    Args:
        item_uuid (UUID):
        new_parent_uuid (UUID):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[ApiChangeParentItemV1ItemsItemUuidParentNewParentUuidPutResponseApiChangeParentItemV1ItemsItemUuidParentNewParentUuidPut, HTTPValidationError]]
    """

    kwargs = _get_kwargs(
        item_uuid=item_uuid,
        new_parent_uuid=new_parent_uuid,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    item_uuid: UUID,
    new_parent_uuid: UUID,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Optional[
    Union[
        ApiChangeParentItemV1ItemsItemUuidParentNewParentUuidPutResponseApiChangeParentItemV1ItemsItemUuidParentNewParentUuidPut,
        HTTPValidationError,
    ]
]:
    """Api Change Parent Item

     Rename exising item.

    Args:
        item_uuid (UUID):
        new_parent_uuid (UUID):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[ApiChangeParentItemV1ItemsItemUuidParentNewParentUuidPutResponseApiChangeParentItemV1ItemsItemUuidParentNewParentUuidPut, HTTPValidationError]
    """

    return (
        await asyncio_detailed(
            item_uuid=item_uuid,
            new_parent_uuid=new_parent_uuid,
            client=client,
        )
    ).parsed
